
import os

from mkdocs.plugins import BasePlugin
from mkdocs.config import config_options
from mkdocs_expose_page_metadata.metadata_page import MetadataPage


class ExposePageMetadataPlugin(BasePlugin):

    # CONSTANTS

    _METADATA_PAGE_PATH_CONFIG = "metadata_page_path"
    _METADATA_PAGE_NAME_CONFIG = "metadata_page_name"
    _METADATA_PAGE_ENCODE_CONFIG = "metadata_page_encode"

    # ATTRIBUTES

    config_scheme = (
        (_METADATA_PAGE_PATH_CONFIG, config_options.Type(str, default=MetadataPage.PATH)),
        (_METADATA_PAGE_NAME_CONFIG, config_options.Type(str, default=MetadataPage.NAME)),
        (_METADATA_PAGE_ENCODE_CONFIG, config_options.Type(str, default=MetadataPage.ENCODE)),
    )

    # EVENTS

    def on_pre_build(self, config):
        self.metadata = MetadataPage(
            path=os.path.join(config['site_dir'], self.config[self._METADATA_PAGE_PATH_CONFIG]),
            name=self.config[self._METADATA_PAGE_NAME_CONFIG],
            encode=self.config[self._METADATA_PAGE_ENCODE_CONFIG]
        )

    def on_page_context(self, context, **kwargs):
        "Metadata Page Creation"
        _PAGE = context['page']
        self.metadata.append_content_from(_PAGE)

    def on_post_build(self, config, **kwargs):
        "Metadata Page Building"
        self.metadata.save()



import json

from mkdocs.contrib.search.search_index import ContentParser
from mkdocs.structure.pages import Page

def find_toc_by_id(toc, _id):
    for toc_item in toc:
        if toc_item.id == _id:
            return toc_item
        toc_item_r = find_toc_by_id(toc_item.children, _id)
        if toc_item_r is not None:
            return toc_item_r

def get_subtitle_url_from(page: Page, parser: ContentParser, subtitle: str):

    _subtitle = string_remove(text=subtitle, old="#", strip=True)

    for section in parser.data:
        toc_item = find_toc_by_id(page.toc, section.id)
        toc_item_title = string_remove(text=toc_item.title if toc_item else "", old="#", strip=True)
        if toc_item_title == _subtitle:
            return page.url + toc_item.url

    return None

def string_remove(text: str, old: str, count: int=-1, strip: bool=False):
    return string_replace(text=text, old=old, count=count, strip=strip)

def string_replace(text: str, old: str, new:str="", count: int=-1, strip: bool=False):
    if not text:
        return text
    result = text.replace(old, new if new else "", count if count else -1)
    return result.strip() if strip else result

def create_parser_from(page: Page):
    _PARSER = ContentParser()
    _PARSER.feed(page.content)
    _PARSER.close()
    return _PARSER

def to_json(object):
    return json.dumps(object, default=lambda o: o.__dict__, separators=(',', ':'), sort_keys=True)

def dict_merge(base: dict={}, new: dict={}) -> dict:
    _new = new if new else {}
    _base = base if base else {}
    for key in _new.keys():
        _base[key] = dict_merge_value(_base, key, _new[key])
    return _base

def dict_merge_value(base: dict={}, key: str=None, value=None):
    _base = base if base else {}
    if key and key in _base:
        _value = _base[key] if isinstance(_base[key],list) else [_base[key]]
        _value.append(value)
        return _value
    else:
        return value



from mkdocs_expose_page_metadata.metadata import Metadata

class MetadataPageEntry:

    @property
    def title(self) -> str:
        return self._TITLE

    @property
    def url(self) -> str:
        return self._URL

    @property
    def meta(self) -> Metadata:
        return self._META

    def __init__(self, title: str, url: str, meta: dict=None, annotations: list[str] = []):
        self._TITLE = title
        self._URL = url
        self._META = Metadata(content=meta, annotations=annotations)

    def to_dict(self):
        return {
            'title': self.title,
            'url': self.url,
            'meta': self.meta.content
        }



import os

from mkdocs import utils
from mkdocs.structure.pages import Page
from mkdocs_expose_page_metadata.metadata_page_entry import MetadataPageEntry
from mkdocs_expose_page_metadata.helpers import to_json
from mkdocs_expose_page_metadata.metadata_parser import MetadataParser

class MetadataPage:

    PATH = "metadata"
    NAME = "metadata.json"
    ENCODE = "utf-8"

    def __init__(self, path: str, name: str, encode: str):
        self._PATH = path if path else self.PATH
        self._NAME = name if name else self.NAME
        self._ENCODE = encode if encode else self.ENCODE
        self._CONTENT = list[MetadataPageEntry]()

    def append_content_from(self, page: Page):

        if page.meta:
            entry = MetadataPageEntry(title=page.title, url=page.url, meta=page.meta)
            self._CONTENT.append(entry)

        parser = MetadataParser(page)

        for entry in parser.entries:
            if (not entry.url):
                raise Exception(f"URL not found for subtitle \"{_SUBTITLE}\" on page \"{page.title}\"")
            self._CONTENT.append(entry)


    def save(self):
        _ENDPOINT = os.path.join(self._PATH, self._NAME)
        _CONTENT = self.__str__()
        utils.write_file(_CONTENT.encode(self._ENCODE), _ENDPOINT)

    def __repr__(self) -> str:
        return to_json([entry.to_dict() for entry in self._CONTENT])



from mkdocs_expose_page_metadata.helpers import string_remove, dict_merge, dict_merge_value

class Metadata:

    @property
    def content(self) -> dict:
        return self._CONTENT

    def __init__(self, content: dict=None, annotations: list[str]=[]):
        self._CONTENT = dict_merge(content, self._extract_content_from(annotations))

    def _extract_content_from(self, annotations: list[str]) -> dict:
        content = {}
        for annotation in annotations:
            key, value = self._extract_entry_from(annotation)
            if key:
                content[key] = dict_merge_value(content, key, value)
        return content

    def _extract_entry_from(self, annotation: str) -> (str, str):

        # [meta]:<key> (value)
        if annotation.startswith("[meta]:"):
            _annotation = string_remove(text=annotation, old="[meta]:", count=1, strip=True)
            key = _annotation[1:_annotation.find(">")]
            value = _annotation[_annotation.find(">")+1:].strip()[1:-1]
            return (key, value)

        # [key]:<> (value)
        if annotation.startswith(f"[{annotation[1:annotation.find(']')]}]:<>"):
            key = annotation[1:annotation.find(']')]
            value = annotation[annotation.find('<>')+2:].strip()[1:-1]
            return (key, value)

        print(f'This metadata annotation is invalid: {annotation} ! It will not be displayed in the metadata page!')
        return None



from setuptools import setup

setup(
    name='mkdocs-expose-page-metadata',
    version='0.1.3',
    description='This plug-in exposing all pages metadata in a single path!',
    long_description='See https://github.com/herberton/mkdocs-expose-page-metadata for more details.',
    keywords='mkdocs metadata',
    url='https://github.com/herberton/mkdocs-expose-page-metadata',
    author='Herberton Candido Souza',
    author_email='herberton@gmail.com',
    license='Apache-2.0',
    python_requires='>=3',
    packages=['mkdocs_expose_page_metadata'],
    install_requires=[
        'mkdocs>=1.1.2'
    ],
    entry_points={
        'mkdocs.plugins': [
            'mkdocs-expose-page-metadata = mkdocs_expose_page_metadata.plugin:ExposePageMetadataPlugin',
        ]
    }
)


import re

from mkdocs.structure.pages import Page
from mkdocs_expose_page_metadata.helpers import create_parser_from, get_subtitle_url_from, string_remove
from mkdocs_expose_page_metadata.metadata_page_entry import MetadataPageEntry

class MetadataParser:

    _PAGE_MARKDOWN_REGEX = re.compile(r"(#+\s*.*\s*(\[.*\]:<.*>\s*\(.*\)\s*)+)")
    _PAGE_SUBTITLE_REGEX = re.compile(r"(#+\s*.*)")
    _PAGE_ANNOTATIONS_REGEX = re.compile(r"(\[.*\]:<.*>\s*\(.*\)\s*)")

    @property
    def entries(self) -> list[MetadataPageEntry]:
        return self._ENTRIES

    def __init__(self, page: Page):

        self._ENTRIES = []

        parser = create_parser_from(page)

        matches = self._PAGE_MARKDOWN_REGEX.finditer(page.markdown)
        for match in matches:

            text = match.group(0)

            subtitle = string_remove(text=self._PAGE_SUBTITLE_REGEX.search(text).group(0), old="#", strip=True)
            url = get_subtitle_url_from(page=page, parser=parser, subtitle=subtitle)
            annotations = [annotation.group(0).strip() for annotation in self._PAGE_ANNOTATIONS_REGEX.finditer(text)]

            self._ENTRIES.append(MetadataPageEntry(title=subtitle, url=url, annotations=annotations))

